/*-
 * Copyright (c) 2008 Michael Telahun Makonnen <mtm@FreeBSD.Org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/types.h>
#include <sys/queue.h>

#include <pthread.h>

/*
 * Internal representation of a packet.
 */
struct fs_pkt {
	STAILQ_ENTRY(fs_pkt) fp_link;
	struct sockaddr_in   fp_saddr;	/* divert(4) address/port of packet */
	socklen_t	     fp_salen;	/* size in bytes of fp_addr */
	uint16_t fp_class;		/* our classification */
	u_char	 *fp_pkt;		/* raw packet from divert(4) */
	size_t	 fp_pktlen;		/* size in bytes of packet */
};

STAILQ_HEAD(pkt_head, fs_pkt);

/*
 * Structure on which incomming/outgoing packets are queued.
 */
struct fs_queue {
	pthread_cond_t	fq_condvar;	/* signaled when pkts are available */
	pthread_mutex_t fq_mtx;		/* syncronization mutex */
	struct pkt_head fq_pkthead;	/* queue head */
	int fq_maxsz;			/* max size (in packets) of queue */
	int fq_size;			/* current size */
};
